
#define EXPORT_SYMTAB

#if defined(CONFIG_MODVERSIONS) && !defined(MODVERSIONS)
#define MODEVERSIONS
#endif

#ifdef MODVERSIONS
#include <linux/modversions.h>
#endif
#include <linux/module.h>
#include <linux/init.h>
#include <linux/proc_fs.h>
#include <linux/delay.h>
#include <linux/timer.h>
#include <linux/wait.h>
#include <asm/uaccess.h>
#include "register.h"
#include "led.h"

#ifdef DEBUG
#define PRINTK(format,argument...) printk(format,##argument)
#else
#define PRINTK(format,argument...)
#endif

MODULE_LICENSE("GPL");

static struct semaphore led_sem;
static spinlock_t       led_lock;
static ssize_t proc_read_led_fops(struct file *filp,
                                 char *buf,size_t count , loff_t *offp)
{
	char tmp[512]="";
	int len=0,i=0;
	unsigned long           flagLock=0;
	
	if(*offp!=0)
		return 0;
	
	spin_lock_irqsave (&led_lock, flagLock);
	for(i=0;led_data[i].gpio_pin!=0;i++){	
		len+=sprintf(tmp+len,"GPIO%02d %d\n",i,led_data[i].gpio_pin&GPIO_DATA_OUTPUT?0:1);
	}
	spin_unlock_irqrestore (&led_lock, flagLock);
	
	len=strlen(tmp)+1;
	copy_to_user(buf,tmp,len);
	PRINTK("<1>read...\n");
	*offp = len;
	return len;
}
static ssize_t proc_read_mac_fops(struct file *filp,
                                 char *buf,size_t count , loff_t *offp)
{
	char tmp[24]="";
	char dev_addr[6];
	int len=0;
	if(kerSysGetMacAddress(dev_addr,2)!=0)
		return 0;

	len=sprintf(tmp,"%02X:%02X:%02X:%02X:%02X:%02X",dev_addr[0],dev_addr[1],dev_addr[2],dev_addr[3],dev_addr[4],dev_addr[5]);
	copy_to_user(buf,tmp,len);
	*offp = len;
	return len;
}


static int led_atoi( char *name)
{
	int val = 0;
  for(;;name++)
	{
		switch(*name)
		{
			case '0'...'9':
				val = val*10+(*name - '0');
				break;
			default:
				return val;
		}
	}
}

void led_on(unsigned long gpio,int logic)
{
		
	if(logic > 0)
		GPIO_DATA_OUTPUT |= gpio;
	else
		GPIO_DATA_OUTPUT &= ~gpio;
				
}

void led_off(unsigned long gpio,int logic)
{
	if(logic > 0)
		GPIO_DATA_OUTPUT &= ~gpio;
	else
		GPIO_DATA_OUTPUT |= gpio;
}

static void led_init(unsigned long gpio,int init,int logic)
{	
	GPIO_DATA_ENABLE |= gpio;
//	GPIO_DATA_DIR &= ~gpio;
	if(init)
		led_on(gpio,logic);
	else
		led_off(gpio,logic);
}

static void led_reverse(unsigned long gpio)
{
	if(GPIO_DATA_OUTPUT & gpio)
		GPIO_DATA_OUTPUT &= ~gpio;
	else
		GPIO_DATA_OUTPUT |= gpio;
}


static struct timer_list led_slow_timer;
static struct timer_list led_fast_timer;

static void led_blink_timer(unsigned long speed)
{
	struct led_data_s *led;
	unsigned long           flagLock=0;
	
	spin_lock_irqsave (&led_lock, flagLock);
	for(led=led_data;led->gpio_pin!=0;led++){
		if(led->count > 0 && led->speed==speed ){
			led_reverse(led->gpio_pin);
			led->count--;
			
			if(led->count == 0){
				/* last stat is off*/
				if(led->last_stat==0)
					led_off(led->gpio_pin,led->logic);
				/* last stat is on */
				else if(led->last_stat==1)
					led_on(led->gpio_pin,led->logic);
			}
		}
	}
	
	spin_unlock_irqrestore (&led_lock, flagLock);
	
	if( speed == LED_FAST){	
		led_fast_timer.data=speed;
       		led_fast_timer.expires=jiffies+FAST_INTERVAL;
	       	add_timer(&led_fast_timer);
	}else{
		led_slow_timer.data=speed;
       		led_slow_timer.expires=jiffies+SLOW_INTERVAL;
	       	add_timer(&led_slow_timer);
	}

}

void led_blink(unsigned long gpio,int count ,int speed,int last_stat)
{
	struct led_data_s *led;
	unsigned long           flagLock=0;
	
	spin_lock_irqsave (&led_lock, flagLock);

	for(led=led_data;led->gpio_pin!=0;led++){
		if(led->gpio_pin & gpio ){
			led->speed=speed;
			led->count=2*count;
			led->last_stat=last_stat;
		}
	}
	spin_unlock_irqrestore (&led_lock, flagLock);
}


static ssize_t proc_write_led_fops(struct file *filp,const char *buffer,
                                     size_t count , loff_t *offp)
{
	char line[10];
	
	copy_from_user(line,buffer,count);
	line[count]=0x00;
	
	PRINTK("<0>echo in==%s\n",line);
	
	if(count<1) 
		return count;
		
	down_interruptible(&led_sem);	
	
	switch(line[0]){
		/* upgrade */
		case 'u':
			/* 2006.03.10 changed to meet NR's Spec. */
			led_on(GPIO_POWER,-1);
			led_blink(GPIO_POWER,led_atoi(&line[1]),LED_SLOW,1);
			break;
		/* test */
		case 't':
			led_on(GPIO_TEST,-1);		
			mdelay(300);
			led_off(GPIO_TEST,-1);
			break;
		/* reboot */
		case 'r':
			led_on(GPIO_TEST,-1);
			led_on(GPIO_POWER,-1);
			mdelay(1000);
			led_on(GPIO_RESET,-1);
			//led_off(GPIO_TEST,-1);
			break;
		/* write flash */
		case 'w':
			/* 2006.03.10 changed to meet NR's Spec. */
			led_off(GPIO_POWER,-1);
			led_blink(GPIO_POWER,led_atoi(&line[1]),LED_FAST,1);
			break;
		/* adsl */
		case 'a':
			if(line[1]=='1' && line[2]=='\0')
				led_on(GPIO_ADSL_ACT,-1);
			else if(line[1]=='0')
				led_off(GPIO_ADSL_ACT,-1);
			else
				led_blink(GPIO_ADSL_ACT,led_atoi(&line[1]),LED_FAST,1);
			break;
		/* sync */
		case 's':
			led_blink(GPIO_ADSL_DOWN,led_atoi(&line[1]),LED_SLOW,0);
			break;	
		/* restore to default */
		case 'd':
			led_blink(GPIO_TEST,led_atoi(&line[1]),LED_SLOW,0);
			break;		
		/* gpio test */
		case 'g':
			{
				unsigned long gpio=(1<<led_atoi(&line[1]));
				GPIO_DATA_ENABLE |= gpio;
				if(strstr(&line[1],"on"))
					led_on(gpio,-1);
				else
					led_off(gpio,-1);
			}
			break;
	}
	

	up(&led_sem);	

	return count;
}

static struct file_operations led_fops = {
        read: proc_read_led_fops,
        write: proc_write_led_fops,
};
static struct file_operations mac_fops = {
        read: proc_read_mac_fops,
};

static struct proc_dir_entry *led_file;
static struct proc_dir_entry *mac_file;

int my_led_init(void)
{
	int i=0;
	PRINTK("init...\n");
	led_file=create_proc_entry("led",0666,&proc_root);
	led_file->owner = THIS_MODULE;
	led_file->proc_fops = &led_fops;
	mac_file=create_proc_entry("mac",0666,&proc_root);
	mac_file->owner = THIS_MODULE;
	mac_file->proc_fops = &mac_fops;

	led_init(GPIO_RESET,0,-1);	
	for(i=0;led_data[i].gpio_pin!=0;i++){	
		led_init(led_data[i].gpio_pin,led_data[i].init,led_data[i].logic);	
	}
	
	init_timer(&led_slow_timer);

	led_slow_timer.data=LED_SLOW;
       	led_slow_timer.expires=jiffies+SLOW_INTERVAL;
	led_slow_timer.function=led_blink_timer;
	add_timer(&led_slow_timer);
	
	init_timer(&led_fast_timer);

	led_fast_timer.data=LED_FAST;
       	led_fast_timer.expires=jiffies+FAST_INTERVAL;
	led_fast_timer.function=led_blink_timer;
        add_timer(&led_fast_timer);
	
	sema_init(&led_sem,1);
	
	return 0;
}
void  my_led_cleanup(void)
{
	remove_proc_entry("led",&proc_root);
	del_timer(&led_fast_timer);
	del_timer(&led_slow_timer);
	PRINTK("<0>cleanup...\n");
}
#if 0
module_init(led_init_module);
module_exit(led_cleanup_module);
#endif	
EXPORT_SYMBOL(led_blink);
